/*
%Jun 11 2010
%Chih-Yuan Yang EECS UC Merced
%need to convert this function to C, otherwise, Matlab runs it too slow
*/
#include "mex.h"
#include <math.h>
#include <memory.h>



void Convolute(const mxArray *H , const size_t width_in, const size_t height_in , const double Bk_GauVar , mxArray **ppH_out)
{
	double pi = 3.14159265359;
    int width = (int)width_in;
    int height = (int)height_in;
    double *dH = mxGetPr(H);
 
    double sigma = sqrt( Bk_GauVar );
    *ppH_out = mxCreateNumericMatrix( height, width , mxDOUBLE_CLASS , mxREAL);
    double *dHBk = mxGetPr( *ppH_out );
    for( int i = 0 ; i <height ; i++ )
    {
        for( int j=0 ; i<width ; j++ )
        {
            double YCoor = i-0.5;
            double XCoor = j-0.5;
            double EffectiveRadius = sigma * 4;
            double MostLeftTopSourcePoint_X = XCoor - EffectiveRadius;
            double MostLeftTopSourcePoint_Y = YCoor - EffectiveRadius;
            double MostRightBottomSourcePoint_X = XCoor + EffectiveRadius;
            double MostRightBottomSourcePoint_Y = YCoor + EffectiveRadius;
            //trim
            if( MostLeftTopSourcePoint_X < 0.5 )
                MostLeftTopSourcePoint_X = 0.5;
            
            if( MostLeftTopSourcePoint_Y < 0.5 )
                MostLeftTopSourcePoint_Y = 0.5;
            
            if( MostRightBottomSourcePoint_X > width - 0.5 )
                MostRightBottomSourcePoint_X = width - 0.5;
            
            if( MostRightBottomSourcePoint_Y > height - 0.5 )
                MostRightBottomSourcePoint_Y = height - 0.5;
            
            //Convert Coordinate to grid index
            int MostLeftTopSourcePoint_ColIdx = (int)(MostLeftTopSourcePoint_X - 0.5);
            int MostLeftTopSourcePoint_RowIdx = (int)(MostLeftTopSourcePoint_Y - 0.5);
            int MostRightBottomSourcePoint_ColIdx = (int)(MostRightBottomSourcePoint_X - 0.5);
            int MostRightBottomSourcePoint_RowIdx = (int)(MostRightBottomSourcePoint_Y - 0.5);
            double WeightSum = 0;
            double IntensitySum = 0;
            for(int r = MostLeftTopSourcePoint_RowIdx ; r<=MostRightBottomSourcePoint_RowIdx ; r++ )
            {
                for(int c = MostLeftTopSourcePoint_ColIdx ; c<=MostRightBottomSourcePoint_ColIdx ; c++ )
                {
                    double SrcY = (double)r - 0.5;
                    double SrcX = (double)c - 0.5;
                    double distSqr = (SrcY - YCoor)*(SrcY - YCoor) + (SrcX - XCoor)*(SrcX - XCoor);
                    //modified here
                    if( distSqr < Bk_GauVar * 16.0)
                    {
                        double weight = exp(-distSqr/(2*Bk_GauVar))/(2*pi*Bk_GauVar);
                        WeightSum += weight;
                        IntensitySum += weight * dH[r*width+c];     //is this format coorect?
                    }
                }
            }
            dHBk[i*width+j] = IntensitySum / WeightSum ;            
        }
    }
}

//          1              0      1 
//function HBk = Convolute(H, Bk_GauVar )
void mexFunction(int nlhs, mxArray *plhs[],
				 int nrhs, const mxArray *prhs[])
{
	size_t width = mxGetN( prhs[0] );
    size_t height = mxGetM( prhs[0] );
	double *dTemp = mxGetPr( prhs[1] );
	double Bk_GauVar = *dTemp;
	Convolute(prhs[0], width, height, Bk_GauVar, &plhs[0]);
}
