%Chih-Yuan Yang, EECS, UC Merced
%File Created: 29 Sep 2010
%Last modified: 29 Sep 2010
%Change the cluster setting, make some vectors overlap, try if the change can produce better group sparsity result

function CoefMatrix = FindCoef3( Para , Iter)
    CopyString = ['copy ' mfilename '.m ' Para.TempDataFolder];
    dos(CopyString);
    clear CopyString
    
    TempFolder = Para.TempDataFolder;
    SaveName = Para.SaveName;
    tStart = now;

    if ispc
        addpath('Lib\spgl1-1.7');
    elseif isunix
        addpath('Lib/spgl1-1.7');       %unix's folder is case-sensitive
    end
    codesize = Para.DictionarySize;

    load([TempFolder SaveName '_ClusterResultAllMatchArray_Avg' num2str(Para.AvgSignalNumPerCluster) '.mat'], 'AllMatchArray', 'ClusterNum');

    LoadData = load([TempFolder SaveName '_DlDh' num2str(Iter-1) '.mat']);
    Dictionary = [LoadData.DlDh];
    clear LoadData;
    
    LoadData = load([TempFolder SaveName '_XlXh.mat']);
    XlXh = LoadData.XlXh;
    SignalNum = size(XlXh,2);
    clear LoadData;
    
    CoefMatrix_Sum = zeros( codesize, SignalNum );      %out of memory here
    CoefMatrix_ClusterCount = zeros(1,SignalNum);
    BaseSigma = Para.BaseSigma;
    opts = spgSetParms('verbosity',0,'iterations',Para.IterNumForGroupSparseCoding);   %1000
    OptimizationSum = 0;
    for CluIdx = 1:ClusterNum
        MatchArray = AllMatchArray(:,CluIdx);
        InClusterNum = sum(MatchArray);%AllInClusterNum(CluIdx);
        sigma = BaseSigma * sqrt(InClusterNum);     %Feb 17, add sqrt, adding should be correct
        InClusterGroupIdx = MatchArray';
        Y = XlXh(:,InClusterGroupIdx);
        if ~isempty(Y)
            Coef = spg_mmv(Dictionary,Y,sigma,opts);     %the form is min ||X||1,2 s.t. ||AX-B||2,2 <= sigma
            CoefMatrix_Sum(1:codesize, InClusterGroupIdx) = CoefMatrix_Sum(1:codesize, InClusterGroupIdx) + Coef;
            CoefMatrix_ClusterCount(InClusterGroupIdx) = CoefMatrix_ClusterCount(InClusterGroupIdx) + 1;
            disp( ['Find Coef ...' num2str(CluIdx) ' / ' num2str(ClusterNum)]);
            DiffMatrix = Y - Dictionary * Coef;
            DiffSquare = DiffMatrix.*DiffMatrix;
            GroupSignalDiff = sum(DiffSquare(:));
            CoefSquare = Coef .* Coef;
            CoefpqNorm = sum(sqrt(sum(CoefSquare,2)));
            OptimizationSum = OptimizationSum + GroupSignalDiff + CoefpqNorm;
        end
        %Computer the optimization sum
        pause(0.1);
    end
    CoefMatrix = CoefMatrix_Sum ./ repmat(CoefMatrix_ClusterCount , codesize , 1);
    OptSum_Coef = OptimizationSum;
    WriteToInfoFile(OptSum_Coef, Para);
    save([TempFolder SaveName '_CoefMatrix_Iter' num2str(Iter) '.mat'] , 'CoefMatrix' );
    rmpath('lib\spgl1-1.7');
    tEnd = now;
    CoefFindTime = ['(Iter' num2str(Iter) ')' datestr(tEnd-tStart, 'dd:HH:MM:SS')];
    WriteToInfoFile( CoefFindTime , Para );
end

